import {
  ConflictException,
  forwardRef,
  Inject,
  Injectable,
} from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';
import { Equipo } from '../equipo/entity/equipo.entity';
import { EquipoTipoEntrada } from './entity/equipo-tipo-entrada.entity';
import { TipoEntrada } from '../institucion-tipo-entrada/entity/tipo-entrada.entity';
import { FullInformacionEquipoTipoEntradaView } from './entity/views/full-informacion-equipo-tipo-entrada.view';
import { InformacionEquipoTipoEntradaView } from './entity/views/informacion-equipo-tipo-entrada.view';
import { EquipoService } from '../equipo/equipo.service';
import { InstitucionTipoEntradaService } from '../institucion-tipo-entrada/institucion-tipo-entrada.service';

@Injectable()
export class EquipoTipoEntradaService {
  constructor(
    @InjectRepository(EquipoTipoEntrada)
    private repository: Repository<EquipoTipoEntrada>,
    @InjectRepository(FullInformacionEquipoTipoEntradaView)
    private fullInformacionEquipoTipoEntradaView: Repository<FullInformacionEquipoTipoEntradaView>,
    @InjectRepository(InformacionEquipoTipoEntradaView)
    private informacionEquipoTipoEntradaView: Repository<InformacionEquipoTipoEntradaView>,
    @Inject(forwardRef(() => EquipoService))
    private equipoService: EquipoService,
    private institucionTipoEntradaService: InstitucionTipoEntradaService,
  ) {}

  async create(
    id_equipo: number | Equipo,
    id_tipo_entrada: number | TipoEntrada,
  ) {
    const equipo =
      typeof id_equipo === 'number'
        ? await this.equipoService.findInfoEquipoById(id_equipo)
        : id_equipo;
    const tipoEntrada =
      typeof id_tipo_entrada === 'number'
        ? await this.institucionTipoEntradaService.findTipoEntradaById(
            id_tipo_entrada,
          )
        : id_tipo_entrada;

    return this.informacionEquipoTipoEntrada(
      equipo.id_equipo,
      tipoEntrada.id_tipo_entrada,
    )
      .then((existeEquipoTipoEntrada) => {
        if (existeEquipoTipoEntrada)
          throw new ConflictException(
            'Este tipo de entrada ya fue asignado a este equipo.',
          );
        return this.repository.save(
          this.repository.create({ equipo, tipoEntrada }),
        );
      })
      .then((equipoTipoEntrada) => ({
        message: `Se asignó el tipo de entrada: ${tipoEntrada.tipo_entrada}, al equipo con número de inventario: ${equipo.numero_inventario}.`,
        equipoTipoEntrada,
      }));
  }

  async createCargaMasiva(equipo: Equipo, tipoEntrada: TipoEntrada) {
    return this.repository
      .save(this.repository.create({ equipo, tipoEntrada }))
      .then((equipoTipoEntrada) => ({
        message: `Se asignó el tipo de entrada: ${tipoEntrada.tipo_entrada}, al equipo con número de inventario: ${equipo.numero_inventario}.`,
        equipoTipoEntrada,
      }));
  }

  delete(id_equipo_tipo_entrada: number) {
    return this.findById(id_equipo_tipo_entrada)
      .then((equipoTipoEntrada) => this.repository.remove(equipoTipoEntrada))
      .then((_) => ({
        message: 'Se eliminó correctamente el tipo de entrada de este equipo.',
      }));
  }

  findById(id_equipo_tipo_entrada: number) {
    return this.repository
      .findOne({ where: { id_equipo_tipo_entrada } })
      .then((equipoTipoEntrada) => {
        if (!equipoTipoEntrada)
          throw new ConflictException('No existe este id equipo tipo entrada.');
        return equipoTipoEntrada;
      });
  }

  findFullInfoEquipoTipoEntradaAllByIdEquipo(id_equipo: number) {
    return this.fullInformacionEquipoTipoEntradaView
      .find({ where: { id_equipo } })
      .then((infoEquipoTipoEntradas) => {
        const equipoProgramas: EquipoTipoEntrada[] = [];

        for (let i = 0; i < infoEquipoTipoEntradas.length; i++)
          equipoProgramas.push(
            this.repository.create({
              id_equipo_tipo_entrada:
                infoEquipoTipoEntradas[i].id_equipo_tipo_entrada,
              tipoEntrada: {
                id_tipo_entrada: infoEquipoTipoEntradas[i].id_tipo_entrada,
                tipo_entrada: infoEquipoTipoEntradas[i].tipo_entrada,
              },
            }),
          );
        return equipoProgramas;
      });
  }

  informacionEquipoTipoEntrada(id_equipo: number, id_tipo_entrada: number) {
    return this.informacionEquipoTipoEntradaView.findOne({
      where: { id_equipo, id_tipo_entrada },
    });
  }
}
